/*
    AQTRONIX C++ Library
    Copyright 2003-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// IPRangeList.cpp: implementation of the CIPRangeList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IPRangeList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* Changelog
 * 2006.07.30 Added Enumerate(CStringArray&) and ToString()
 * 2005.10.02 Added AddIPRange(CIPRange&)
 *            Changed parameters CString to LPCTSTR
 * 2004.01.16 First version of class finished
 * 2003.10.15 Created class for MailKnight
 */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIPRangeList::CIPRangeList()
{

}

CIPRangeList::~CIPRangeList()
{
	RemoveAll();
}

bool CIPRangeList::AddIPRange(CIPRange& Range)
{
	RangeList.AddTail(&Range);
	return true;
}

bool CIPRangeList::AddIPRange(LPCTSTR Range)
{
	CIPRange* p = new CIPRange;
	if(p->SetIPRange(Range)){
		RangeList.AddTail(p);
		return true;
	}else{
		delete p;
		return false;
	}
}

bool CIPRangeList::Enumerate(CStringList &List)
{
	RemoveAll();
	POSITION pos;
	pos = List.GetHeadPosition();
	while(pos!=NULL){
		CIPRange* p = new CIPRange;
		if(p->SetIPRange(List.GetNext(pos))){
			RangeList.AddTail(p);
		}else{
			delete p;
		}
	}
	return true;
}

bool CIPRangeList::Enumerate(CStringArray &List)
{
	RemoveAll();
	int size = List.GetSize();
	for(int i=0; i<size; i++){
		CIPRange* p = new CIPRange;
		if(p->SetIPRange(List[i])){
			RangeList.AddTail(p);
		}else{
			delete p;
		}
	}
	return true;
}

bool CIPRangeList::IsInList(LPCTSTR IP)
{
	/*
	 * return true if IP is in the list
	 */
	POSITION pos;
	CIPRange* ipr = NULL;
	pos = RangeList.GetHeadPosition();
	while (pos != NULL){
		ipr = (CIPRange*)RangeList.GetNext(pos);
		if(ipr->IsInRange(IP)){
			return true;
		}
	}
	return false;
}

bool CIPRangeList::IsInList(CIPAddress& IP)
{
	/*
	 * return true if IP is in the list
	 */
	POSITION pos;
	CIPRange* ipr = NULL;
	pos = RangeList.GetHeadPosition();
	while (pos != NULL){
		ipr = (CIPRange*)RangeList.GetNext(pos);
		if(ipr->IsInRange(IP)){
			return true;
		}
	}
	return false;
}

bool CIPRangeList::IsEmpty()
{
	return (RangeList.GetHeadPosition()==NULL);
}

void CIPRangeList::RemoveAll()
{
	POSITION pos;
	pos = RangeList.GetHeadPosition();
	while(pos!=NULL){
		delete RangeList.GetNext(pos);
	}
	RangeList.RemoveAll();
}

CString CIPRangeList::ToString(LPCTSTR Separator)
{
	CString ret("");
	POSITION pos;
	pos = RangeList.GetHeadPosition();
	while(pos!=NULL){
		CIPRange* p = (CIPRange*) RangeList.GetNext(pos);
		ret += p->ToString();
		if(pos!=NULL){
			ret += Separator;
		}
	}
	return ret;
}

