/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// ODBCConnection.cpp: implementation of the CODBCConnection class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#define _WIN32_WINNT 0x501//CoInitializeEx()
//#include "objbase.h"	//CoInitializeEx()
#include "ODBCConnection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/* CHANGELOG
 *  2005.07.06 Class created
 */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CODBCConnection::CODBCConnection(LPCTSTR DSN):IsOpen(false),IsCreated(false),lf("C:\\Debug\\ODBCConnection\\ODBC.log")
{
	SetDSN(DSN);
	CreateObject();
}

CODBCConnection::~CODBCConnection()
{
	DestroyObject();
}

bool CODBCConnection::CreateObject()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());//otherwise assertion failed

	try{

		if(IsCreated){
			SetError("Already created");
			return false;
		}else{
			HRESULT hr;
			//TODO: CoInitializeEx(NULL) failed
			hr = CoInitializeEx(NULL,COINIT_MULTITHREADED);
			//hr = CoInitialize(NULL);
			if(FAILED(hr)){
				SetError(CString("Could not initialize COM: ") + CProcessHelper::GetErrorDescription(hr));
				return false;
			}else{
				SetError("CreateInstance()");
				//TODO: crashes on this line below
				hr = Connection.CreateInstance(__uuidof(ado20::Connection));
				SetError("CreateInstance() done");
				if(FAILED(hr)){
					switch(hr){
					case CLASS_E_NOAGGREGATION:
						SetError("No aggregation!");
						break;
					case REGDB_E_CLASSNOTREG:
						SetError("Class not registered!");
						break;
					default:
						SetError("Failed to create connection object");
						break;
					}
					return false;
				}else{
					IsCreated = true;
					return true; //connection created
				}
			}
		}

	}catch(...){
		SetError("CreateObject() failed");
		return false;
	}
}

bool CODBCConnection::DestroyObject()
{
	try{

		if(IsCreated){
			//close if open
			Close();

			//destroy object
			Connection.Release();
			Connection = NULL;
			
			CoUninitialize();

			return true;
		}else{
			SetError("Could not destroy: no connection object");
			return false;
		}

	}catch(...){
		SetError("DestroyObject() failed");
		return false;
	}
}

bool CODBCConnection::Open(LPCTSTR DSN)
{
	//close existing connection
	Close();

	//set new DSN
	SetDSN(DSN);

	//open connection
	return Open();
}

bool CODBCConnection::Open()
{
	//create if not already created
	if(!IsCreated)
		CreateObject();
	
	if(IsOpen){
		//already open
		SetError("Already open");
		return false;
	}else{
		//only open if DSN is known
		if(DSN==""){
			SetError("Invalid DSN");
			return false;
		}else{
			return (IsOpen = OpenConnection());
		}
	}
}

bool CODBCConnection::OpenConnection()
{
	try{

		if(IsCreated){
			//Connection->PutConnectionTimeout(1);
			if(FAILED(Connection->Open(_bstr_t(DSN),_bstr_t(""),_bstr_t(""),NULL))){
				//open failed
				SetError("Could not open connection");
				return false;
			}else{
				//open successful
				return true;
			}
		}else{
			//no connection object
			SetError("Could not open connection: no connection object");
			return false;
		}

	}catch(...){
		//fatal error
		SetError("OpenConnection() failed");
		return false;
	}
}

void CODBCConnection::Close()
{
	//close if open
	if(IsOpen){
		IsOpen = !CloseConnection();
	}
}

bool CODBCConnection::CloseConnection()
{
	try{

		if(IsCreated){
			if(Connection->State){
				if(FAILED(Connection->Close())){
					//close failed
					SetError("Could not close connection");
					return false;
				}else{
					//close successful
					return true;
				}
			}else{
				//already closed (state==0)
				SetError("Could not close connection: already closed");
				return false;
			}
		}else{
			//no connection object
			SetError("Could not close connection: no connection object");
			return false;
		}

	}catch(...){
		//fatal error
		SetError("CloseConnection() failed");
		return false;
	}
}

void CODBCConnection::SetDSN(LPCTSTR DSN)
{
	//set new DSN string
 	CODBCConnection::DSN = DSN;
}

bool CODBCConnection::Execute(LPCTSTR SQL)
{
	try{

		if(IsCreated){
			if(IsOpen){
				Connection->Execute((_bstr_t)SQL,NULL,ado20::adCmdText);
				//execute successful
				return true;
			}else{
				//connection not open
				SetError("Could not execute: connection closed");
				return false;
			}
		}else{
			//no connection object
			SetError("Could not execute: no connection object");
			return false;
		}

	}catch(...){
		//fatal error
		SetError("Execute() failed");
		return false;
	}
}

CString CODBCConnection::GetError()
{
	//return error messages
	return Error;
}

void CODBCConnection::SetError(LPCTSTR ErrMsg)
{
	//set error message
	Error = ErrMsg;
	lf.WriteLine(ErrMsg);
}
