/*
    AQTRONIX C++ Library
    Copyright 2004-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// HTTPFirewallSettings.h: interface for the CHTTPFirewallSettings class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTTPFIREWALLSETTINGS_H__58F563F1_7F52_4F31_9119_AA03546009B0__INCLUDED_)
#define AFX_HTTPFIREWALLSETTINGS_H__58F563F1_7F52_4F31_9119_AA03546009B0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "FirewallSettings.h"
#include "WebAgents.h"

class CHTTPFirewallSettings : public CFirewallSettings  
{
public:
	CString ToXML();
	bool ReadFromFile(LPCTSTR fn);
	void LoadDefaultsHTTPVersions(CStringList& List);
	void LoadDefaultsAuthAccounts(CStringList& List);
	void LoadDefaultsAuthPasswords(CStringList& List);
	void LoadDefaultsSQLInjection(CStringList& List);
	void LoadDefaultsUnixCommands(CStringList& List);
	bool Update();
	void ReadAgentsFile(LPCTSTR Path);
	virtual void LoadAgents(){};

	virtual bool CheckConstraints();

	bool WriteToActiveDirectory();
	bool WriteToRegistry();
	bool ReadFromRegistry();
	bool ReadFromActiveDirectory();
	bool ProcessFile(LPCTSTR fn);
	void LoadDefaults();
	CHTTPFirewallSettings();
	virtual ~CHTTPFirewallSettings();

	//SCANNING ENGINE
	class EngineParameters{
	public:
		bool ScanNonSecurePort;
		bool ScanSecurePort;
		bool AllowLateScanning;
		_StringList ExcludedInstances;
	}Engine;

	//RESPONSE HANDLING
	class HTTPResponseParameters{
	public:
		bool DropConnection;
		bool UseStatus;
		CString Status;
		CString RedirectURL;
		bool Redirect;
		CString Contents;
		bool Directly;
	}HTTPResponse;
	
	//LOGGING
	class HTTPLoggingParameters{
	public:
		bool LogUserAgent;
		bool Log_HTTP_X_FORWARDED_FOR;
		bool Log_HTTP_VIA;
		bool LogServerErrors;
		bool LogClientErrors;
	}HTTPLogging;

	//CONNECTION
	class ConnectionParameters{
	public:
		_StringList MonitorAddresses;
		_StringList DenyAddresses;
		_StringCache RequestsLimit;
	} Connection;

	//AUTHENTICATION
	class AuthenticationParameters{
	public:
		bool ScanAccountAllEvents;
		bool ScanExcludedInstances;
		bool DenyBlankPasswords;
		bool DenySamePasswordAsUsername;
		_StringList DenyDefaultPasswords;
		_StringCache DenyBruteForceAttack;
		bool DenySystemAccounts;
		_StringList AllowAccounts;
		_StringList DenyAccounts;
	} Authentication;

	//REQUEST LIMITS
	class LimitsParameters{
	public:
		bool LimitContentLength;
		bool LimitURL;
		bool LimitQueryString;
		bool LimitHTTPVersion;
		CString MaxContentLength; //CString object!!!
		int MaxURL;
		int MaxQueryString;
		int MaxHTTPVersion;
		_StringList AllowHTTPVersions;
		_StringMap MaxHeaders;
	}Limits;

	//URL SCANNING
	class URLParameters{
	public:
		bool DenyEncodingExploits;
		bool RFCCompliantURL;
		bool RFCCompliantHTTPURL;
		bool UseRawScan;
		bool DenyMultipleCGI;
		bool DenyEscaping;
		bool DenyAlternateStream;
		bool DenyBackslash;
		bool DenyTrailingDotInDir;
		bool DenyParentPath;
		bool DenyHighBitShellcode;
		CString DenyCharacters;
		_StringList DenySequences;
		_StringList AllowURL;
	}URL;

	//QUERYSTRING
	class QueryStringParameters{
	public:
		bool UseRawScan;
		bool DenyHighBitShellcode;
		bool DenyEncodingExploits;
		bool DenyDirectoryTraversal;
		bool DenySQLInjection;
		_StringList DenySequences;
	} QueryString;

	//MAPPED PATH
	class _Path{
	public:
		bool DenyBackspace;
		bool DenyCarriageReturn;
		bool DenyNewLine;
		bool DenyEscaping;
		bool DenyParentPath;
		CString DenyCharacters;
		bool DenyDot;
		_StringList AllowPaths;
	}Path;

	//REQUESTED FILE
	class RequestedFilenameParameters{
	public:
		bool UseRawScan;
		bool DenyDefaultDocument;
		CString DenyCharacters;
		_StringList DenyFiles;
		_StringList MonitorFiles;
	}Filename;

	//EXTENSION
	class ExtensionsParameters{
	public:
		_StringList AllowExtensions;
		_StringList DenyExtensions;
	}Extensions;

	//HEADERS
	class HeadersParameters{
	public:
		_StringList DenyHeaders;
		_StringList AllowContentTypes;
		bool DenyCookieEncodingExploits;
		bool DenyCookieSQLInjection;
		bool RFCCompliantHostHeader;
		CString ServerHeader;
		bool ChangeServerHeader;
		bool RemoveServerHeader;
		bool DenyHighBitShellcode;
		bool DenyEncodingExploits;
		bool DenyDirectoryTraversal;
		bool DenySQLInjection;
		_StringList DenySequences;
	}Headers;
	
	//HEADERS - USER AGENT
	class UserAgentParameters{
	public:
		_StringList DenyUserAgents;
		_StringList DenySequences;
		bool DenyNonRFC;
		bool DenyEmpty;
		CString RequireCharacter;
	}UserAgent;

	//METHODS
	class VerbsParameters{
	public:
		_StringList DenyVerbs;
		_StringList AllowVerbs;
	}Verbs;

	//GLOBAL FILTER CAPABILITIES
	class GlobalParameters{
	public:
		bool IsInstalledAsGlobalFilter;
		bool DenyPostHighBitShellcode;
		bool DenyPostEncodingExploits;
		bool DenyPostDirectoryTraversal;
		bool DenyPostSQLInjection;
		_StringList DenyPostSequences;
	}Global;
	
	//SQL INJECTION
	CStringList SQLKeyWords;

	//AGENTS DATABASE
	class _Agents{
	public:
		CWebAgents AgentList;
		bool BlockDataMiningCommercial;
		bool BlockDataMiningPublic;
		bool BlockDownloadManagers;
		bool BlockEmailHarvesting;
		bool BlockGuestbookSpammers;
		bool BlockHackTools;
		bool BlockImageDownloaders;
		bool BlockIndexing;
		bool BlockMonitoring;
		bool BlockOfflineBrowsers;
		bool BlockOtherBad;
		bool BlockTrademark;
		bool BlockValidationTools;
		bool BlockLinkChecking;
		bool BlockBrowsers;
		bool BlockMediaPlayers;
		bool BlockProxies;
		bool BlockAdware;
		bool BlockBrowserExtensions;
		bool BlockSpyware;
		bool BlockEditing;
		bool BlockDevice;
		bool BlockNewsFeed;
		bool BlockSearchEngines;
		bool BlockFilteringSoftware;
		bool BlockSoftwareComponent;
	}Agents;

	class _Referrer{
	public:
		bool UseScan;
		bool RFCCompliantURL;
		bool RFCCompliantHTTPURL;
		bool DenyEncodingExploits;
		bool DenyHotLinking;
		CStringList HotLinkingFileExtensions;
		_StringList HotLinkingAllowDomains;
		_StringList HotLinkingDenyDomains;
		bool HotLinkingUseHostHeader;
		bool HotLinkingDenyBlankReferrer;
		CString DenyCharacters;
		bool DenyHighBitShellcode;
		_StringList DenySequences;
	}Referrer;

protected:
	CString GetXMLUserAgent(CHTTPFirewallSettings& d);
	CString GetXMLReferrer(CHTTPFirewallSettings& d);
	CString GetXMLLogging(CHTTPFirewallSettings& d);
	CString GetXMLHTTPVerbs(CHTTPFirewallSettings &d);
	CString GetXMLSQLInjection(CHTTPFirewallSettings& d);
	CString GetXMLGlobalFilterCapabilities(CHTTPFirewallSettings& d);
	CString GetXMLQuerystringScanning(CHTTPFirewallSettings& d);
	CString GetXMLHTTPHeaders(CHTTPFirewallSettings& d);
	CString GetXMLRequestedExtension(CHTTPFirewallSettings& d);
	CString GetXMLRequestedFile(CHTTPFirewallSettings& d);
	CString GetXMLMappedPath(CHTTPFirewallSettings& d);
	CString GetXMLURLScanning(CHTTPFirewallSettings& d);
	CString GetXMLRequestLimits(CHTTPFirewallSettings& d);
	CString GetXMLAuthentication(CHTTPFirewallSettings& d);
	CString GetXMLConnectionControl(CHTTPFirewallSettings& d);
	CString GetXMLResponseHandling(CHTTPFirewallSettings& d);
	CString GetXMLScanningEngine(CHTTPFirewallSettings& d);
	bool WriteToFileXML(LPCTSTR fn);
	bool WriteToFileINI(LPCTSTR fn);
	bool ParseXML(CString& key, CString& val);
	bool ReadFromFileINI(LPCTSTR fn);
	bool ReadFromFileURLScan(LPCTSTR fn);
};

#endif // !defined(AFX_HTTPFIREWALLSETTINGS_H__58F563F1_7F52_4F31_9119_AA03546009B0__INCLUDED_)
